local revision = tonumber(string.sub("$Revision: 30 $", 12, -3))
local rss = _G.RShamanStat
if rss.revision < revision then rss.revision = revision end

--Monitor and record player gear, stat, and buff changes

local playerState = 
{
	ascendance = nil,
	ascendanceAura = false,
	ancestralGuidance = nil,
	guidanceAura = false,
	tidalWaves = nil,
	healingBonus = 0,
	damageBonus = { [1] = 0.0, [2] = 0.0, [3] = 0.0, [4] = 0.0, [5] = 0.0, [6] = 0.0, [7] = 0.0 },
	masteryBonus = 0.0,
	criticalChance = { [1] = 0.0, [2] = 0.0, [3] = 0.0, [4] = 0.0, [5] = 0.0, [6] = 0.0, [7] = 0.0 },
	criticalMultiplier = rss.CRITICAL_MULTIPLIER,
	aaSetBonusT15 = false,
	spiritLinkGUID = nil,
	lastCast = { },
	lastFlameShock = { combatEvent = "SPELL_DAMAGE" },
	busy = 0,
	busyRing = nil,
	busyCounts = 0,
	busyPointer = 0,
	busyInterval = 1.0/rss.WINDOW_RATE,
	busyElapsed = 0.0,
}
rss.playerState = playerState;

local healthTable = { };
rss.healthTable = healthTable;

function rss.CreateCopyRecord()
	local record = { };
	record.rawHealing = 0.0;
	record.incrementalInt = 0.0;
	record.incrementalMastery = 0.0;
	record.incrementalCritical = { 0.0 , 0.0 };
	record.effectiveCopies = 0.0;
	return record;
end

-- deal with healing copy abilities
-- we assume that every 1% increase in our raw (not effective) heals during the duration will mean a corresponding 1% increase in the effective (not raw) copied heals
function rss.ProcessCopyRecord(record, fight)
	if fight and record.rawHealing > 0 then
		fight.incrementalIntellect = fight.incrementalIntellect + record.effectiveCopies*(record.incrementalInt/record.rawHealing);
		record.incrementalInt = 0.0;
		fight.incrementalMastery = fight.incrementalMastery + record.effectiveCopies*(record.incrementalMastery/record.rawHealing);
		record.incrementalMastery = 0.0;
		for i = 1, 2 do
			fight.incrementalCritical[i] = fight.incrementalCritical[i] + record.effectiveCopies*(record.incrementalCritical[i]/record.rawHealing);
			record.incrementalCritical[i] = 0.0;
		end
		record.rawHealing = 0.0;
		record.effectiveCopies = 0.0;
	end
end

function rss.ResetPlayerState()
	rss.CheckGear();
	rss.ClearHealthTable();
	playerState.lastCast = { };
	playerState.spiritLinkGUID = nil;
	playerState.ascendance = nil;
	playerState.ancestralGuidance = nil;
	rss.InitializeRing();
end

function rss.MonitorState(fight)
	playerState.healingBonus = GetSpellBonusHealing();
	for i = 1, 7 do
		playerState.damageBonus[i] = GetSpellBonusDamage(i);
	end
	playerState.masteryBonus = 0.03 * GetMastery();
	for i = 1, 7 do
		playerState.criticalChance[i] = GetSpellCritChance(i);
	end
	local tidalWavesUp = false;
	local ascendanceUp = false;
	local guidanceUp = false;
	for i = 1, 40 do
		local id = select(11,UnitAura("player", i));
		if id then
			if id == rss.AURA_TIDAL_WAVES then
				tidalWavesUp = true;
			elseif id == rss.AURA_ASCENDANCE then
				ascendanceUp = true;
			elseif id == rss.AURA_ANCESTRAL_GUIDANCE then
				guidanceUp = true;
			end
		else
			break;
		end
	end
	playerState.tidalWaves = tidalWavesUp;
	if ascendanceUp and not playerState.ascendanceAura then
		if playerState.ascendance then
			rss.ProcessCopyRecord(playerState.ascendance, fight);
		end
		playerState.ascendance = rss.CreateCopyRecord();
	end
	playerState.ascendanceAura = ascendanceUp;
	
	if guidanceUp and not playerState.guidanceAura then
		if playerState.ancestralGuidance then
			rss.ProcessCopyRecord(playerState.ancestralGuidance, fight);
		end
		playerState.ancestralGuidance = rss.CreateCopyRecord();
	end
	playerState.guidanceAura = guidanceUp;
	
	return playerState;
end

-- Busy ring maintenance
function rss.InitializeRing()
	playerState.busyRing = { };
	local busyRing = playerState.busyRing;
	for i = 0, rss.WINDOW_BINS-1 do
		busyRing[i] = 0;
	end
	playerState.busyCounts = 0;
	playerState.busyPointer = 0;
	playerState.busyElapsed = 0.0;
end

function rss.UpdateRing()
	playerState.busy = (UnitCastingInfo("player") or GetSpellCooldown(rss.LIGHTNING_SHIELD) > 0 or UnitChannelInfo("player")) and 1 or 0;
	playerState.busyCounts = playerState.busyCounts - playerState.busyRing[playerState.busyPointer];
	playerState.busyRing[playerState.busyPointer] = playerState.busy;
	playerState.busyCounts = playerState.busyCounts + playerState.busy;
	playerState.busyPointer = (playerState.busyPointer + 1) % rss.WINDOW_BINS;
end

-- health table maintenance
function rss.ClearHealthTable()
	if UnitHealthMax("player") > 0 then
		healthTable = { [UnitGUID("player")] = { maxHealth = UnitHealthMax("player") , currentHealth = UnitHealth("player") } };
	else
		healthTable = { };
	end
	rss.healthTable = healthTable;
end
-- respond to HEALTH_CHANGED and MAX_HEALTH_CHANGED events for party/raid members
function rss.UpdateHealthTable(...)
	local unitID = select(1,...);
	if unitID == nil then return; end
	if UnitIsPlayer(unitID) or UnitPlayerOrPetInParty(unitID) or UnitPlayerOrPetInRaid(unitID) then
		rss.UpdateUnitHealth(unitID);
	end
end
-- general routine to add any unit we can heal to the health table
function rss.UpdateUnitHealth(unitID)
	if not UnitCanAssist("player", unitID) then return; end
	local unitGUID = UnitGUID(unitID);
	if unitGUID == nil then return; end
	local unitHealthTable = healthTable[unitGUID];
	if unitHealthTable == nil then
		unitHealthTable = { };
		healthTable[unitGUID] = unitHealthTable;
	end
	unitHealthTable.maxHealth = UnitHealthMax(unitID);
	unitHealthTable.currentHealth = UnitHealth(unitID);
	if unitHealthTable.maxHealth < 1 then
		healthTable[unitGUID] = nil;
	end
end

-- check for a crit-enhancing metagem
function rss.CheckMeta()
	local gem1, gem2, gem3 = GetInventoryItemGems(GetInventorySlotInfo("HeadSlot")); -- head slot
	if tContains(rss.CRITICAL_BONUS_GEMS, gem1) or 
	   tContains(rss.CRITICAL_BONUS_GEMS, gem2) or 
	   tContains(rss.CRITICAL_BONUS_GEMS, gem3) then
		playerState.criticalMultiplier = rss.CRITICAL_MULTIPLIER * 1.03;
	else
		playerState.criticalMultiplier = rss.CRITICAL_MULTIPLIER;
	end
end

-- Find the item level of something equipped in a slot (including upgrades, which is why this isn't as easy as it should be)
function rss.GetItemLevel(slot)
	local g=MyTooltipScan or CreateFrame("GameTooltip","MyTooltipScan",UIParent,"GameTooltipTemplate")
	g:SetOwner(UIParent,"ANCHOR_NONE")
	g:SetInventoryItem("player",slot)
	for i=1,g:NumLines() do
		local line = _G["MyTooltipScanTextLeft"..i]:GetText();
		local ilvl = line:match('%d%d%d$');
		if ilvl then
			return tonumber(ilvl);
		end
	end
end

-- Scan equipped gear for tier 15 pieces and the Lei-Shen trinket
function rss.CheckGear()
	rss.CheckMeta();

	playerState.aaSetBonusT15 = false;
	local n15 = 0;
	for slotName, tierItems in pairs(rss.SET_BONUS_T15) do
		local equipID = GetInventoryItemID("player", GetInventorySlotInfo(slotName));
		if equipID and tContains(tierItems, equipID) then
			n15 = n15 + 1;
		end
	end
	if n15 >= 4 then
		playerState.aaSetBonusT15 = true;
	end
	
	local chaliceLevel = nil;
	for slotName, trinkets in pairs(rss.LIGHTNING_IMBUED_CHALICE) do
		local equipID = GetInventoryItemID("player", GetInventorySlotInfo(slotName));
		if equipID and tContains(trinkets, equipID) then
			chaliceLevel = rss.GetItemLevel(GetInventorySlotInfo(slotName));
			break;
		end
	end
	-- fill the appropriate base heal determined by the item level
	if chaliceLevel and rss.CHALICE_BASE_HEAL[chaliceLevel] then
		local baseHeal = rss.CHALICE_BASE_HEAL[chaliceLevel];
		local coeff = rss.SPELLS[rss.PROC_RESTORATION_INFINITE]["SPELL_HEAL"];
		coeff[1] = baseHeal;
		coeff[2] = baseHeal;
	end
end
